//
//  ViewController.swift
//  TimeTecBLEDemo
//
//  Created by Nicholas Chong on 16/08/2019.
//  Copyright © 2019 TimeTec Cloud Sdn Bhd. All rights reserved.
//  Swift 5.0 Demo
//

import UIKit
import TimeTecBLE

class ViewController: UIViewController {
    
    @IBOutlet weak var rightBarButton: UIBarButtonItem!
    @IBOutlet weak var licenseView: UIView!
    @IBOutlet weak var contentView: UIView!
    @IBOutlet weak var tableView: UITableView!
    
    @IBOutlet weak var textFieldLicense: UITextField!
    
    var bleLists: [BLE] = []
    
    var macAddressList: [String: String] = [:]
    
    override func viewDidLoad() {
        super.viewDidLoad()
        
        self.title = "TimeTec BLE Demo"
        initialize()
    }
    
    override func touchesBegan(_ touches: Set<UITouch>, with event: UIEvent?) {
        self.view.endEditing(true)
    }
    
    func initialize() {
        
        let defaultList = UserDefaults.standard.dictionary(forKey: "macAddressList")
        if bleLists.isEmpty {
            if defaultList != nil {
                macAddressList = defaultList as! [String : String]
                for (macAddress, bleType) in self.macAddressList {
                    let item = BLE(bleType: BLEType(rawValue: bleType) ?? BLEType.BLE2, macAddress: macAddress)
                    bleLists.append(item)
                }
                contentView.isHidden = false
            } else {
                contentView.isHidden = true
            }
        }
        
        tableView.tableFooterView = UIView()
        
    }
    
    // MARK: - Event Actions
    @IBAction func buttonTopRight(_ sender: UIBarButtonItem) {
        let alertController = UIAlertController(title: "Add MAC Address", message: "BLE2: 34:81:F4:50:08:25\nBLE16: 34:81:F4:15:1D:07\nNote: (:) is optional", preferredStyle: .alert)
        let confirmAction = UIAlertAction(title: "Add", style: .default) { (_) in
            
            if let txtField = alertController.textFields?.first, let text = txtField.text {
                
                // TODO: - MAC Address Authentication
                let authentication = TimeTecBLE.shared.authenticateMacAddress(text)
                if authentication.result {
                    
                    let defaultList = UserDefaults.standard.dictionary(forKey: "macAddressList")
                    if defaultList != nil {
                        
                        self.macAddressList = defaultList as! [String : String]
                        self.macAddressList[text.filter { !":".contains($0) }] = authentication.bleType
                        UserDefaults.standard.set(self.macAddressList, forKey: "macAddressList")
                        
                        self.bleLists.removeAll()
                        for (macAddress, bleType) in self.macAddressList {
                            let item = BLE(bleType: BLEType(rawValue: bleType) ?? BLEType.BLE2, macAddress: macAddress)
                            self.bleLists.append(item)
                        }
                        
                    } else {
                        
                        let item = BLE(bleType: BLEType(rawValue: authentication.bleType) ?? BLEType.BLE2, macAddress: text.filter { !":".contains($0) }
                        )
                        self.macAddressList[text.filter { !":".contains($0) }] = authentication.bleType
                        UserDefaults.standard.set(self.macAddressList, forKey: "macAddressList")
                        self.bleLists.append(item)
                        
                    }
                    self.tableView.reloadData()
                    
                } else {
                    
                    let alert = UIAlertController(title: "Failure", message: "MAC Address invalid or license not verified", preferredStyle: UIAlertController.Style.alert)
                    alert.addAction(UIAlertAction(title: "OK", style: UIAlertAction.Style.default, handler: {(_: UIAlertAction!) in
                        // actions
                    }))
                    self.present(alert, animated: true, completion: nil)
                    
                }
                
            }
        }
        let cancelAction = UIAlertAction(title: "Cancel", style: .cancel) { (_) in }
        alertController.addTextField { (textField) -> Void in
            textField.placeholder = "Enter MAC Address here..."
            textField.keyboardType = .asciiCapable
            textField.autocapitalizationType = .allCharacters
        }
        alertController.addAction(confirmAction)
        alertController.addAction(cancelAction)
        self.present(alertController, animated: true, completion: nil)
    }
    
    @IBAction func buttonVerify(_ sender: UIButton) {
        self.view.endEditing(true)
        self.showSpinner(onView: self.view)
        self.rightBarButton.isEnabled = false
        
        // TODO: - License Authentication
        TimeTecBLE.shared.verifyLicense(licenseId: self.textFieldLicense.text ?? "") { (result) in
            
            self.removeSpinner()
            
            DispatchQueue.main.async {
                let alert = UIAlertController(title: result ? "Success" : "Failure", message: result ? "License verified" : "License invalid", preferredStyle: UIAlertController.Style.alert)
                alert.addAction(UIAlertAction(title: "OK", style: UIAlertAction.Style.default, handler: {(_: UIAlertAction!) in
                    UIView.animate(withDuration: 0.5, animations: {
                        if self.bleLists.isEmpty {
                            self.contentView.isHidden = result ? false : true
                        } else {
                            self.contentView.isHidden = false
                        }
                        self.rightBarButton.isEnabled = true
                    })
                }))
                self.present(alert, animated: true, completion: nil)
            }
            
        } // end result
    }
    override func prepare(for segue: UIStoryboardSegue, sender: Any?) {
        
        if segue.identifier == "listingsegue" {
            if let indexPath = self.tableView.indexPathForSelectedRow {
                let controller = segue.destination as! ListingViewController
                controller.bleData = self.bleLists[indexPath.row]
            }
        }
        
    }
    
}

// MARK: - UITableViewDataSource, UITableViewDelegate
extension ViewController: UITableViewDataSource, UITableViewDelegate {
    
    func tableView(_ tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
        return bleLists.count
    }
    
    func tableView(_ tableView: UITableView, cellForRowAt indexPath: IndexPath) -> UITableViewCell {
        
        let cell = tableView.dequeueReusableCell(withIdentifier: "cell", for: indexPath) as! TableViewCell
        cell.labelMacAddress.text = "\(indexPath.row + 1)) \(bleLists[indexPath.row].macAddress)"
        cell.labelType.text = bleLists[indexPath.row].bleType.rawValue
        
        return cell
        
    }
    
    func tableView(_ tableView: UITableView, didSelectRowAt indexPath: IndexPath) {
        self.performSegue(withIdentifier: "listingsegue", sender: self)
    }
    
    func tableView(_ tableView: UITableView, canEditRowAt indexPath: IndexPath) -> Bool {
        return true
    }
    
    func tableView(_ tableView: UITableView, commit editingStyle: UITableViewCell.EditingStyle, forRowAt indexPath: IndexPath) {
        
        if editingStyle == .delete {
            macAddressList[bleLists[indexPath.row].macAddress] = nil
            UserDefaults.standard.set(self.macAddressList, forKey: "macAddressList")
            
            DispatchQueue.main.async {
                self.bleLists.remove(at: indexPath.row)
                self.tableView.beginUpdates()
                self.tableView.deleteRows(at: [indexPath], with: .automatic)
                self.tableView.endUpdates()
            }
        }
        
    }
    
}

// MARK: Extension UIVIewController
var indicatorView: UIView?
extension UIViewController {
    func showSpinner(onView : UIView) {
        let spinnerView = UIView.init(frame: onView.bounds)
        spinnerView.backgroundColor = UIColor.init(red: 0.5, green: 0.5, blue: 0.5, alpha: 0.5)
        let ai = UIActivityIndicatorView.init(style: .whiteLarge)
        ai.startAnimating()
        ai.center = spinnerView.center
        
        DispatchQueue.main.async {
            spinnerView.addSubview(ai)
            onView.addSubview(spinnerView)
        }
        
        indicatorView = spinnerView
    }
    
    func removeSpinner() {
        DispatchQueue.main.async {
            indicatorView?.removeFromSuperview()
            indicatorView = nil
        }
    }
}
